#ccbasic compiler
import sys

#to use: ccbasic [code.txt] [output.txt] [mode (normal or debug)]

#Registers T00-T63 are available
#Registers B08-B63 are available (not sure if this will continue to be allowed though)

#############################
#     Instructions          #
#############################
#Assign statements: LET T# = $$ [+-/* $$] [+-/* $$] . . . etc
#GOTO [label]
#GOSUB [label]
#RETURN
#EXIT
#SYMBOL T# = [label]
#IF VAR COND VAR/LITERAL
#    ...code...
#ELIF VAR COND VAR/LITERAL
#    ...code...
#ELSE
#    ...code...
#ENDIF

#B00-B07 are reserved due to GOSUB stack


if len(sys.argv) < 3:
    print "Correct format: ccbasyc.py [input_filename] [output_filename] [optional:mode]"
input_file  = sys.argv[1]
output_file = sys.argv[2] 
mode = 'normal'
if len(sys.argv) > 3:
    mode = sys.argv[3]

s_type = ['S0', 'S1', 'S2', 'S3', 'S4', 'S5', 'S6', 'S7']
a_type = ['A0', 'A1', 'A2', 'A3', 'A4', 'A5', 'A6', 'A7']
op_type = ['+', '-', '/', '*', '<<', '>>', '&&', '||']


instr_list=[]
cur_cmd=[]
label_pos = {}
symbols = {}

#Open the file and read it into raw_code
infile  = open(input_file,"r")
outfile = open(output_file,"w")
raw_code = infile.readlines()
intermediate_code = []

#if mode=='debug':
#    print "Initial code:"
#    print raw_code

#Build the symbol table
for line in raw_code:
    cur_cmd=line.split()
    if len(cur_cmd) != 0:
        if cur_cmd[0]=='SYMBOL':
            symbols[cur_cmd[1]] = cur_cmd[3]
        
if mode=='debug':
    print "Building symbol table . . . "
    print "Symbols:"
    print symbols

#Expand all of the code into simplified RTL format
# and convert IF statements into a temporary format

for line in raw_code:
    cur_cmd=line.split()

    #This just detects whitespace lines
    if len(cur_cmd)==0:
        nop = 0 

    #Define a SYMBOL (i.e. SYMBOL COUNT = B37)
    elif cur_cmd[0] == 'SYMBOL':
        nop = 0
        #do nothing

    #Define a LABEL (i.e. LABEL START) - for jumping to elsewhere in the code
    elif cur_cmd[0] == 'LABEL':      #We just detected a label
        intermediate_code.append('LABEL ' + cur_cmd[1])

    #LET statements (i.e. LET B# = B# + B# - 7)
    # -> These allow us to perform calculations and
    #    manipulate words
    elif(len(cur_cmd)>1 and cur_cmd[0]=='LET'):     #B-type assign of the form B# = (B# or literal) OP (B# or literal) OP (B# or literal) OP (B# or literal) . . .etc.
        intermediate_code.append("S0 = " + cur_cmd[3])
        for i in range(4,len(cur_cmd),2):            #loop through the remaining operand + variable/literal pairs
            if cur_cmd[i] in op_type:
                intermediate_code.append("S1 = " + cur_cmd[i+1])
                intermediate_code.append("S0 = S0 "+ cur_cmd[i] + " S1")
            else:
                print "ERROR! ERROR!: " + cur_cmd[i] + " is not a recognized operator"
                break
        intermediate_code.append(cur_cmd[1] + " = S0") 

    #GOTO statements - Branch to a LABEL somewhere (i.e. GOTO START)
    elif(cur_cmd[0]=='GOTO'):
        intermediate_code.append("BRANCH " + cur_cmd[1])                #BRANCH to the LABEL


    #GOSUB statements - Branch to a LABEL subroutine somewhere (i.e. GOSUB compute_something_or_other)
    #                   This is presently a fairly bad solution, but it should offer a subroutine stack that is 8 levels deep
    #                   *** This means that B00-B07 cannot be used if GOSUBs are used ***
    elif(cur_cmd[0]=='GOSUB'):
        #Free up B00 to Push onto stack
        intermediate_code.append("A1 = B00")
        intermediate_code.append("A2 = B01")
        intermediate_code.append("A3 = B02")
        intermediate_code.append("A4 = B03")
        intermediate_code.append("A5 = B04")
        intermediate_code.append("A6 = B05")
        intermediate_code.append("A7 = B06")
        intermediate_code.append("B01 = A1")
        intermediate_code.append("B02 = A2")
        intermediate_code.append("B03 = A3")
        intermediate_code.append("B04 = A4")
        intermediate_code.append("B05 = A5")
        intermediate_code.append("B06 = A6")
        intermediate_code.append("B07 = A7")
        #Execute GOSUB (which places return addr in B00)
        intermediate_code.append("GOSUB " + cur_cmd[1])
        #Pop from stack (shift everything back down)
        intermediate_code.append("A1 = B01")
        intermediate_code.append("A2 = B02")
        intermediate_code.append("A3 = B03")
        intermediate_code.append("A4 = B04")
        intermediate_code.append("A5 = B05")
        intermediate_code.append("A6 = B06")
        intermediate_code.append("A7 = B07")
        intermediate_code.append("B00 = A1")
        intermediate_code.append("B01 = A2")
        intermediate_code.append("B02 = A3")
        intermediate_code.append("B03 = A4")
        intermediate_code.append("B04 = A5")
        intermediate_code.append("B05 = A6")
        intermediate_code.append("B06 = A7")

    #RETURN     - Return from a GOSUB
    elif(cur_cmd[0]=='RETURN'):
        intermediate_code.append("BRANCH B00")

    #IF statements - conditional execute:
    #IF VAR COND VAR/LITERAL
    #    ...code...
    #ELIF VAR COND VAR/LITERAL
    #    ...code...
    #ELSE
    #    ...code...
    #ENDIF
    elif(cur_cmd[0]=='IF' or cur_cmd[0]=='ELIF'):               #IF VAR COND VAR/LIT -> Branch to next_thing if (cond) not true
        if cur_cmd[0]=='ELIF':
            intermediate_code.append("GOTO ENDIF")
            intermediate_code.append("ELIF")
        else:
            intermediate_code.append("STARTIF")
        intermediate_code.append("S0 = " + cur_cmd[1])
        intermediate_code.append("S1 = " + cur_cmd[3])
        intermediate_code.append("S0 = S0 - S1")

        if(cur_cmd[2]=='=='):           # ==
            intermediate_code.append("BNE next_thing")
        elif(cur_cmd[2]=='>'):          # >
            intermediate_code.append("BLT next_thing")
        elif(cur_cmd[2]=='<'):          # <
            intermediate_code.append("BGT next_thing")
        elif(cur_cmd[2]=='!='):         # !=
            intermediate_code.append("BEQ next_thing")

    elif(cur_cmd[0]=='ELSE' or cur_cmd[0]=='ENDIF'):
        intermediate_code.append("GOTO ENDIF")             #this is so that the previous IF/ELIF statement doesn't overrun into this one
        intermediate_code.append(cur_cmd[0])    


    #EXIT - Exit the program normally back to the monitor program
    elif(cur_cmd[0]=='EXIT'):
        intermediate_code.append("EXIT")

    #DO - WHILE loops 
    elif(cur_cmd[0]=='DO'):
        print "DO"
        intermediate_code.append("DO")

if mode=='debug':
    print ""
    print "Phase 1: Expanding into RTL-like syntax..."
    for i in intermediate_code:
        print i            

#Now, let's traverse the new list of instructions in reverse-order and 
# replace all of the goto/branch/etc "next_thing"/"endif" statements with useful labels

label_index = 0
label_stacks = []
endif_stack = []
cur_endif   = ''
cur_label_stack = []

for i in range(len(intermediate_code)-1,0,-1):
    new_line = ''
    if(len(intermediate_code[i])>0):
        cur_cmd = intermediate_code[i].split()
        if intermediate_code[i]=='ENDIF':              #detect an ENDIF, assign it a unique label, and then store that label as the "current active ENDIF"
            intermediate_code[i] = 'LABEL ' + 'AUTOGEN_' + str(label_index)
            endif_stack.append('AUTOGEN_' + str(label_index))
            label_index = label_index + 1
            label_stacks.append([])            

        elif intermediate_code[i]=='ELSE' or intermediate_code[i]=='ELIF':
            intermediate_code[i] = 'LABEL ' + 'AUTOGEN_' + str(label_index)
            cur_label_stack = label_stacks.pop()
            cur_label_stack.append('AUTOGEN_' + str(label_index))
            label_stacks.append(cur_label_stack)
            label_index = label_index + 1  
        elif intermediate_code[i]=='GOTO ENDIF':
            cur_endif = endif_stack.pop()       #pop the current ENDIF location so that we can read it
            endif_stack.append(cur_endif)       #then restore it since we're still in the same IF statement
            intermediate_code[i]='GOTO ' + cur_endif
        elif intermediate_code[i]=='STARTIF':
            endif_stack.pop()                   #exit the current IF statement
            label_stacks.pop()
        elif len(cur_cmd)>1 and cur_cmd[1]=='next_thing':
            cur_label_stack = label_stacks.pop()
            if(len(cur_label_stack)>0):             #check if we have any labels waiting (i.e. ELSE, ELIF, etc.)
                cur_cmd[1]=cur_label_stack.pop()
            else:                               #Otherwise just jump to the end
                cur_cmd[1]=endif_stack.pop()    
                endif_stack.append(cur_cmd[1])  #But don't actually pop the endif stack until we get to STARTIF
            label_stacks.append(cur_label_stack)
            for token in cur_cmd:
                new_line = new_line + token + ' '
            
            intermediate_code[i]=new_line
        
if mode=='debug':
    print ""
    print "Phase 2: Generate labels for flow-control statements"
    for i in intermediate_code:
        print i

#The code is now in a simple RTL-style format, everything is properly labeled, and SYMBOLs are still in place
#Let's replace the symbols with their real values
for instr in range(0,len(intermediate_code)):
    cur_cmd = intermediate_code[instr].split()
    for i in range(0,len(cur_cmd)):
        if cur_cmd[i] in symbols:
            cur_cmd[i] = symbols[cur_cmd[i]]        #replace!
    new_cmd = ''
    for i in range(0,len(cur_cmd)):
        new_cmd = new_cmd + cur_cmd[i] + " "
    intermediate_code[instr] = new_cmd

#print the updated instruction list, now sans symbols
if mode=='debug':
    print ""
    print "Phase 3: Symbol replacement"
    for i in intermediate_code:
        print i


#Now let's go through and convert all of the divide instructions into reciprocal + multiply instructions
phase_4_code = []
for instr in range(0,len(intermediate_code)):
    cur_cmd = intermediate_code[instr].split()
    if len(cur_cmd) > 3 and cur_cmd[3]=='/':         #detect divide instructions
        new_cmd = cur_cmd[4] + " = 1 / " + cur_cmd[4]
        phase_4_code.append(new_cmd)
        new_cmd = cur_cmd[0] + " = " + cur_cmd[2] + " * " + cur_cmd[4]
        phase_4_code.append(new_cmd)
    else:
        phase_4_code.append(intermediate_code[instr])

if mode=='debug':
    print ""
    print "Phase 4: Expand divide instructions"
    for i in phase_4_code:
        print i

if mode=='debug':
    print ""
    print "Final code:"
    for i in phase_4_code:
        print i

print ""
print "Writing output..."
for i in phase_4_code:
    outfile.write(i + "\n")
print "Done!"
